from math import *

#G = Genrateur/Gain
#R = resistance
#C = condensateur
#L = bobine inductive
#Uc = tension aux bornes du condensateur
#Ul = tension aux bornes de la bobine
#I = intensite dans le circuit
#Q = charge du condensateur

def module(z): # z complex
        res = sqrt(z.real**2 + z.imag**2)
        return res


#circuit GRC
def GRC_Q(k,E,R,C):
	x = k
	y = C*E*(1-exp(-k/(R*C)))
	return x,y

def GRC_Uc(k,E,R,C):
	x = k
	y = E*(1-exp(-k/(R*C)))
	return x,y

def GRC_I(k,E,R,C):
	x = k
        y = (E/R)*exp(-k/(R*C))
	return x,y
			

#circuit RC (decharge du C)
def RC_I(k,E,R,C):
        x = k
        y = -(E/R)*exp(-k/(R*C))
        return x,y

def RC_Uc(k,E,R,C):
	x = k
	y = E*exp(-k/(R*C))
	return x,y

def RC_Q(k,E,R,C):
        x = k
        y = C*E*exp(-k/(R*C))
        return x,y
			
#circuit GRL
def GRL_I(k,E,R,L):
	x = k
	y = (E/R)*(1-exp(-(R*k)/L))
	return x,y

def GRL_Ul(k,E,R,L):
	x = k
	y = E*exp(-(R*k)/L)
	return x,y
	
#circuit LC (decharge du C)
def LC_Q(k,E,L,C):
	w = 1/sqrt(L*C)
	x = k
	y = C*E*cos(w*k)
	return x,y

def LC_I(k,E,L,C):
	w = 1/sqrt(L*C)
	x = k
	y = w*C*E*sin(w*k)
	return x,y

def LC_Ul(k,E,L,C):
	w = 1/sqrt(L*C)
	x = k
	y = w**2*L*C*E*cos(w*k)
	return x,y

def LC_Uc(k,E,L,C):
	w = 1/sqrt(L*C)
	x = k
	y = E*cos(w*k)
	return x,y

#circuit RL
def RL_I(k,E,R,L):
	x = k
	y = (E/R)*exp(-R*k/L)
	return x,y

def RL_Ul(k,E,R,L):
	x = k
	y = -E*exp(-R*k/L)
	return x,y


#circuit RLC
def RLC_Q(k,E,R,L,C):
	l = R/(2*L)
	w = 1/sqrt(L*C)
	D = (R**2/(4*L**2))-(1/(L*C))
	if D>0:
		x = k
		y = C*E*exp(-l*k)*(cosh(sqrt(l**2-w**2)*k)+(l/(sqrt(l**2-w**2)))*sinh(sqrt(l**2-w**2)*k))
		return x,y
	elif D==0:
		x = k
		y = C*E*(l*k+1)*exp(-l*k)
		return x,y
	else:
		x = k
		y = C*E*exp(-l*k)*(cos(w*k)+(l/w)*sin(w*k))
		return x,y

def RLC_Uc(k,E,R,L,C):
	l = R/(2*L)
	w = 1/sqrt(L*C)
	D = (R**2/(4*L**2))-(1/(L*C))
	if D>0:
		v = sqrt(l**2-w**2)
		x = k
		y = E*exp(-l*k)*(cosh(v*k)+(l/v)*sinh(v*k))
		return x,y
	elif D==0:
		x = k
		y = E*(l*k+1)*exp(-l*k)
		return x,y
	else:
		x = k
		y = E*exp(-l*k)*(cos(w*k)+(l/w)*sin(w*k))
		return x,y

def RLC_I(k,E,R,L,C):
	l = R/(2*L)
	w = 1/sqrt(L*C)
	D = (R**2/(4*L**2))-(1/(L*C))
	if D>0:
		v = sqrt(l**2-w**2)
		x = k
		y = E*exp(-l*k)*(((l**2/v)-v)*sinh(v*k))
		return x,y
	elif D==0:
		x = k
		y = l**2*E*exp(-l*k)*k
		return x,y
	else:
		x = k
		y = E*exp(-l*k)*(w+(l**2/w))*sin(w*k)
		return x,y

def RLC_Ul(k,E,R,L,C):
	l = R/(2*L)
	w = 1/sqrt(L*C)
	D = (R**2/(4*L**2))-(1/(L*C))
	if D>0:
		v = sqrt(l**2-w**2)
		x = k
		y = L*E*((l**2/v)-v)*exp(-l*k)*(v*cosh(v*k)-l*sinh(v*k))
		return x,y
	elif D==0:
		x = k
		y = L*l**2*E*exp(-l*k)*(1-l*k)
		return x,y
	else:
		x = k
		y = L*E*(w+(l**2/w))*exp(-l*k)*(w*cos(w*k)-l*sin(w*k))
		return x,y

# Filtre Passe Bas
def PBas_G(k,R,C):
	w = 1/(R*C)

	x = k
	y = -10*log10(1+(x/w)**2)
	return x,y

def PBas_Phase(k,R,C):
	w = 1/(R*C)

	x = k
	y = - degrees(atan(x/w))
	return x,y

# Filtre Passe Haut
def PHaut_G(k,R,C):
	w = 1/(R*C)

	x = k
	y = 20*log10(x/w) - 10*log10(1+(x/w)**2)
	return x,y

def PHaut_Phase(k,R,C):
	w = 1/(R*C)

	x = k
	y = 90 - degrees(atan(x/w))
	return x,y

# Filtre Passe Bande
def PBande_G(k,R,L,C):
	w = 1/sqrt(L*C)
	Q = (L*w)/R

	x = k
	y = -10*log10(1+Q**2*(x/w - 1/(x/w))**2)
	return x,y

def PBande_Phase(k,R,L,C):
	w = 1/sqrt(L*C)
	Q = (L*w)/R

	x = k
	y = -degrees(atan(Q*(x/w - 1/(x/w))))
	return x,y

# Filtre Coupe Bande
def CBande_G(k,R,L,C,r):
	w = 1/sqrt(L*C)
	Q = L*w/R
	
	x = k
	y = 10*log10(R**2*(1-L*C*x**2)**2 + (R*r*C*x)**2) - 10*log10((r+R*(1-L*C*x**2))**2 + ((R*r*C*x + L*x)**2))
	return x,y

def CBande_Phase(k,R,L,C,r):
	w = 1/sqrt(L*C)
	Q = L*w/R

	x = k
	#y = degrees(atan(Q*(x/w - w/x))) - degrees(atan((Q*(x/w - w/x))/(1+L/(R**2*C))))
	y = degrees(atan((r*R*x)/(1-L*C*x**2))) - degrees(atan(w*(r*R*C+L)/(r+R*(1-L*C*x**2))))
	return x,y

# Circuit Filtre Qcque
def gain_db(x,Z_,Z4):
	x = float(x)
	h = Z4 / (Z4 + Z_)
	try:
		g = module(h)
	except:
		h = complex(h,0)
		g = module(h)
	g_db = 20*log10(g)
	return x,g_db

def phase(x,Z_,Z4):
	x = float(x)
	h = Z4 / (Z4 + Z_)
	try:
		phi = degrees(atan(h.imag/h.real))
	except:
		h = complex(h,0)
		phi = degrees(atan(h.imag/h.real))
	return x,phi
